/*
 * Copyright (C) 2006 Red Hat, Inc.
 *
 * This file is released under the LGPL.
 */

#ifndef __DM_CLOG_TFR_H__
#define __DM_CLOG_TFR_H__

#include <linux/dm-ioctl.h> /* For DM_UUID_LEN */

#define DM_CLOG_TFR_SIZE 1024

#define DM_CLOG_CTR                    1
#define DM_CLOG_DTR                    2
#define DM_CLOG_PRESUSPEND             3
#define DM_CLOG_POSTSUSPEND            4
#define DM_CLOG_RESUME                 5
#define DM_CLOG_GET_REGION_SIZE        6
#define DM_CLOG_IS_CLEAN               7
#define DM_CLOG_IN_SYNC                8
#define DM_CLOG_FLUSH                  9
#define DM_CLOG_MARK_REGION           10
#define DM_CLOG_CLEAR_REGION          11
#define DM_CLOG_GET_RESYNC_WORK       12
#define DM_CLOG_SET_REGION_SYNC       13
#define DM_CLOG_GET_SYNC_COUNT        14
#define DM_CLOG_STATUS_INFO           15
#define DM_CLOG_STATUS_TABLE          16
#define DM_CLOG_IS_REMOTE_RECOVERING  17

#define RQ_TYPE(x) \
	((x) == DM_CLOG_CTR) ? "DM_CLOG_CTR" : \
	((x) == DM_CLOG_DTR) ? "DM_CLOG_DTR" : \
	((x) == DM_CLOG_PRESUSPEND) ? "DM_CLOG_PRESUSPEND" : \
	((x) == DM_CLOG_POSTSUSPEND) ? "DM_CLOG_POSTSUSPEND" : \
	((x) == DM_CLOG_RESUME) ? "DM_CLOG_RESUME" : \
	((x) == DM_CLOG_GET_REGION_SIZE) ? "DM_CLOG_GET_REGION_SIZE" : \
	((x) == DM_CLOG_IS_CLEAN) ? "DM_CLOG_IS_CLEAN" : \
	((x) == DM_CLOG_IN_SYNC) ? "DM_CLOG_IN_SYNC" : \
	((x) == DM_CLOG_FLUSH) ? "DM_CLOG_FLUSH" : \
	((x) == DM_CLOG_MARK_REGION) ? "DM_CLOG_MARK_REGION" : \
	((x) == DM_CLOG_CLEAR_REGION) ? "DM_CLOG_CLEAR_REGION" : \
	((x) == DM_CLOG_GET_RESYNC_WORK) ? "DM_CLOG_GET_RESYNC_WORK" : \
	((x) == DM_CLOG_SET_REGION_SYNC) ? "DM_CLOG_SET_REGION_SYNC" : \
	((x) == DM_CLOG_GET_SYNC_COUNT) ? "DM_CLOG_GET_SYNC_COUNT" : \
	((x) == DM_CLOG_STATUS_INFO) ? "DM_CLOG_STATUS_INFO" : \
	((x) == DM_CLOG_STATUS_TABLE) ? "DM_CLOG_STATUS_TABLE" : \
	((x) == DM_CLOG_IS_REMOTE_RECOVERING) ? \
	"DM_CLOG_IS_REMOTE_RECOVERING" : NULL

struct clog_tfr {
	uint64_t private[2];
	char uuid[DM_UUID_LEN]; /* Ties a request to a specific mirror log */

	int error;              /* Used by server to inform of errors */
	uint32_t originator;    /* Cluster ID of this machine */

	uint32_t seq;           /* Sequence number for request */
	uint32_t request_type;  /* DM_CLOG_* */
	uint32_t data_size;     /* How much data (not including this struct) */
	char data[0];
};

#ifdef __KERNEL__
#define DM_MSG_PREFIX "dm-log-clustered"

int dm_clog_tfr_init(void);
void dm_clog_tfr_exit(void);
int dm_clog_consult_server(const char *uuid, int request_type,
			   char *data, int data_size,
			   char *rdata, int *rdata_size);
#endif

#endif /* __DM_CLOG_TFR_H__ */
